from krrkt.engine.splitter import split_text
from krrkt.engine.fast_layer import FastLayer
from krrkt.engine.deep_layer import DeepLayer
from krrkt.engine.dedup import merge_findings
from krrkt.engine.scorer import calculate_score, score_color, count_words
from krrkt.models import Finding, ProofreadResult
from collections import Counter


_fast_layer = None


def _get_fast_layer():
    global _fast_layer
    if _fast_layer is None:
        _fast_layer = FastLayer()
    return _fast_layer


def _get_provider(model: str | None = None):
    """Get the best available LLM provider.

    Tries Agent SDK first (uses Claude subscription), falls back to Anthropic API.
    """
    resolved_model = model or "claude-sonnet-4-6"

    try:
        from krrkt.providers.agent_sdk_provider import AgentSDKProvider
        return AgentSDKProvider(model=resolved_model)
    except ImportError:
        pass

    try:
        from krrkt.providers.anthropic_provider import AnthropicProvider
        return AnthropicProvider(model=resolved_model)
    except ImportError:
        pass

    raise RuntimeError(
        "No LLM provider available. Install claude-agent-sdk (for subscription) "
        "or set ANTHROPIC_API_KEY (for API)."
    )


def _build_summary(findings: list[dict]) -> dict:
    sections = [
        f.get("section") or f.get("rule", "")
        for f in findings
        if (f.get("section") or f.get("rule")) and f.get("weight", 0) > 0
    ]
    return dict(Counter(sections).most_common())


async def proofread(text: str, skip_llm: bool = False,
                    model: str | None = None) -> ProofreadResult:
    if not text or not text.strip():
        return ProofreadResult(
            score=0, score_color="red", text=text, words=0,
            sentences=0, findings=[], stopwords=0, summary={}
        )

    sentences = split_text(text)

    # Fast layer
    fast_findings = _get_fast_layer().analyze(text)

    # Deep layer (optional)
    deep_findings = []
    if not skip_llm:
        provider = _get_provider(model)
        deep = DeepLayer(provider=provider)
        deep_findings = await deep.analyze(text, already_found=fast_findings)

    # Merge and dedup
    all_findings_raw = merge_findings(fast_findings, deep_findings)

    # Convert to Finding objects
    findings = [Finding(**{k: v for k, v in f.items() if k in Finding.__dataclass_fields__}) for f in all_findings_raw]

    # Score
    score_val = calculate_score(text, [{"weight": f.weight, "penalty": 0} for f in findings])

    return ProofreadResult(
        score=score_val,
        score_color=score_color(score_val),
        text=text,
        words=count_words(text),
        sentences=len(sentences),
        findings=findings,
        stopwords=len([f for f in findings if f.weight > 0]),
        summary=_build_summary(all_findings_raw),
    )



import asyncio
import json
import sys
from pathlib import Path
import typer

from krrkt.engine.pipeline import proofread
from krrkt.engine.scorer import score_color

app = typer.Typer(name="krrkt", help="Russian text quality checker")


def _get_text(text_or_path: str) -> str:
    """If text_or_path is a file path, read the file. Otherwise return as-is."""
    path = Path(text_or_path)
    if path.exists() and path.is_file():
        return path.read_text(encoding="utf-8")
    return text_or_path


def _format_human(result) -> str:
    """Format result for human-readable terminal output."""
    lines = []

    # Score bar
    filled = int(result.score)
    empty = 10 - filled
    bar = "\u2588" * filled + "\u2591" * empty
    color_map = {"green": typer.colors.GREEN, "orange": typer.colors.YELLOW, "red": typer.colors.RED}

    lines.append(f"  {result.score} / 10  {bar}")
    lines.append(f"  {result.words} words, {result.sentences} sentences")
    lines.append("")

    if result.stopwords > 0:
        lines.append(f"  {result.stopwords} stop-words:")

        # Group by severity
        serious = [f for f in result.findings if f.weight >= 80]
        medium = [f for f in result.findings if 0 < f.weight < 80]
        info = [f for f in result.findings if f.weight == 0]

        for f in serious:
            line = f"  \u25a0 \u00ab{f.word}\u00bb {f.rule}"
            if f.description:
                line += f" \u2014 {f.description}"
            lines.append(line)
        for f in medium:
            line = f"  \u25b2 \u00ab{f.word}\u00bb {f.rule}"
            if f.description:
                line += f" \u2014 {f.description}"
            lines.append(line)
        for f in info:
            lines.append(f"  \u25cb \u00ab{f.word}\u00bb {f.rule}")
    else:
        lines.append("  No issues found!")

    return "\n".join(lines)


@app.command()
def check(
    text: str = typer.Argument(help="Text to check or path to file"),
    no_llm: bool = typer.Option(False, "--no-llm", help="Skip LLM analysis (fast layer only)"),
    format: str = typer.Option("human", "--format", "-f", help="Output format: human or json"),
    model: str = typer.Option(None, "--model", "-m", help="LLM model to use"),
):
    """Check text for style issues."""
    content = _get_text(text)
    result = asyncio.run(proofread(content, skip_llm=no_llm, model=model))

    if format == "json":
        import dataclasses
        data = dataclasses.asdict(result)
        del data["text"]  # Don't echo back the full text
        typer.echo(json.dumps(data, ensure_ascii=False, indent=2))
    else:
        typer.echo(_format_human(result))


@app.command()
def score(
    text: str = typer.Argument(help="Text to score"),
    no_llm: bool = typer.Option(False, "--no-llm", help="Skip LLM analysis"),
):
    """Get score only."""
    content = _get_text(text)
    result = asyncio.run(proofread(content, skip_llm=no_llm))
    typer.echo(f"{result.score}/10")


@app.command()
def serve(
    host: str = typer.Option("127.0.0.1", "--host", help="Host to bind"),
    port: int = typer.Option(8080, "--port", "-p", help="Port to bind"),
):
    """Start API server."""
    import uvicorn
    uvicorn.run("krrkt.api.app:app", host=host, port=port, reload=False)


if __name__ == "__main__":
    app()



from fastapi import FastAPI
from pydantic import BaseModel
from krrkt.engine.pipeline import proofread
from krrkt.rules.loader import load_rules

app = FastAPI(title="krrkt", description="Russian text quality checker")


class ProofreadRequest(BaseModel):
    text: str
    skip_llm: bool = False
    model: str | None = None


class FindingResponse(BaseModel):
    start: int
    end: int
    word: str
    rule: str
    section: str = ""
    description: str = ""
    weight: int = 100
    style: str = "red"
    tab: str = "red"
    source: str = "fast"


class ProofreadResponse(BaseModel):
    score: float
    score_color: str
    words: int
    sentences: int
    stopwords: int
    findings: list[FindingResponse]
    summary: dict


@app.post("/api/v1/proofread", response_model=ProofreadResponse)
async def api_proofread(request: ProofreadRequest):
    result = await proofread(
        text=request.text,
        skip_llm=request.skip_llm,
        model=request.model,
    )
    return ProofreadResponse(
        score=result.score,
        score_color=result.score_color,
        words=result.words,
        sentences=result.sentences,
        stopwords=result.stopwords,
        findings=[FindingResponse(**f.__dict__) for f in result.findings],
        summary=result.summary,
    )


@app.get("/api/v1/rules")
def api_rules():
    return load_rules()


@app.get("/api/v1/health")
def api_health():
    return {"status": "ok"}



from krrkt.engine.fast_layer import FastLayer


class TestFastLayerPhrases:
    def setup_method(self):
        self.layer = FastLayer()

    def test_finds_corporate_stamp(self):
        findings = self.layer.analyze("Мы предлагаем широкий спектр услуг.")
        rules = [f["rule"] for f in findings]
        assert "Корпоративный штамп" in rules

    def test_finds_bureaucratese(self):
        findings = self.layer.analyze("В связи с вышеизложенным просим рассмотреть.")
        rules = [f["rule"] for f in findings]
        assert "Канцеляризм" in rules

    def test_finds_time_parasite(self):
        findings = self.layer.analyze("На сегодняшний день всё работает.")
        rules = [f["rule"] for f in findings]
        assert "Паразит времени" in rules

    def test_clean_text_few_findings(self):
        findings = self.layer.analyze("Завод выпускает подшипники.")
        serious = [f for f in findings if f["weight"] > 0]
        assert len(serious) == 0

    def test_returns_positions(self):
        text = "Предлагаем широкий спектр услуг."
        findings = self.layer.analyze(text)
        for f in findings:
            assert "start" in f
            assert "end" in f
            assert "word" in f
            assert f["word"] == text[f["start"]:f["end"]]

    def test_source_is_fast(self):
        findings = self.layer.analyze("Широкий спектр услуг.")
        for f in findings:
            assert f["source"] == "fast"

    def test_no_duplicate_ranges(self):
        findings = self.layer.analyze("На сегодняшний день компания является лидером рынка.")
        ranges = [(f["start"], f["end"]) for f in findings]
        # No two findings should overlap
        for i, (s1, e1) in enumerate(ranges):
            for j, (s2, e2) in enumerate(ranges):
                if i != j:
                    assert not (s1 < e2 and s2 < e1), f"Overlap: {ranges[i]} and {ranges[j]}"

    def test_finds_pleonasm(self):
        findings = self.layer.analyze("Это бесплатный подарок для клиента.")
        rules = [f["rule"] for f in findings]
        assert "Плеоназм" in rules

    def test_finds_zaum_word(self):
        findings = self.layer.analyze("Необходимо верифицировать данные перед интеграцией.")
        rules = [f["rule"] for f in findings]
        assert "Заумное слово" in rules

    def test_finds_new_assessment_words(self):
        findings = self.layer.analyze("Это баснословная цена за феноменальный результат.")
        rules = [f["rule"] for f in findings]
        assert "Необъективная оценка" in rules

    def test_finds_new_introductory_constructions(self):
        findings = self.layer.analyze("Грубо говоря, это казалось бы простая задача.")
        rules = [f["rule"] for f in findings]
        assert "Вводная конструкция" in rules

    def test_finds_new_stamp_phrases(self):
        findings = self.layer.analyze("Мы ставим амбициозные цели и ищем ключ к успеху.")
        patterns = [f["word"] for f in findings]
        assert "амбициозные цели" in patterns or "ключ к успеху" in patterns

    def test_finds_new_bureaucratic_patterns(self):
        findings = self.layer.analyze("Просьба обеспечить выполнение в надлежащем порядке.")
        rules = [f["rule"] for f in findings]
        assert "Канцеляризм" in rules

    def test_finds_stative_verb(self):
        findings = self.layer.analyze("Компания располагается в центре города.")
        rules = [f["rule"] for f in findings]
        assert "Слабый глагол" in rules



from krrkt.engine.fast_layer import FastLayer


class TestFastLayerMorphology:
    def setup_method(self):
        self.layer = FastLayer()

    def test_finds_lemma_in_different_form(self):
        findings = self.layer.analyze("Уникальные решения для бизнеса.")
        rules = [f["rule"] for f in findings]
        assert "Необъективная оценка" in rules

    def test_finds_verb_lemma(self):
        findings = self.layer.analyze("Компания являлась лидером.")
        words = [f["word"].lower() for f in findings]
        assert any("являлась" in w for w in words)

    def test_finds_amplifier_forms(self):
        # "очень" should match as-is (lemma = очень)
        findings = self.layer.analyze("Это очень важный продукт.")
        rules = [f["rule"] for f in findings]
        assert "Усилитель" in rules

    def test_phrase_takes_priority_over_lemma(self):
        # "широкий спектр" as phrase should not also match "широкий" as individual lemma
        text = "Предлагаем широкий спектр услуг."
        findings = self.layer.analyze(text)
        phrase_findings = [f for f in findings if "спектр" in f["word"].lower()]
        assert len(phrase_findings) == 1  # Only the phrase, not individual words

    def test_detects_participle(self):
        # "сделанный" is a PRTF participle
        findings = self.layer.analyze("Сделанный проект был хорош.")
        rules = [f["rule"] for f in findings]
        assert "Возможно, причастие" in rules
        participles = [f for f in findings if f["rule"] == "Возможно, причастие"]
        assert any("сделанный" in f["word"].lower() for f in participles)

    def test_detects_gerund(self):
        # "принимая" is a GRND gerund
        findings = self.layer.analyze("Принимая решение, мы учли факторы.")
        rules = [f["rule"] for f in findings]
        assert "Возможно, деепричастие" in rules
        gerunds = [f for f in findings if f["rule"] == "Возможно, деепричастие"]
        assert any("принимая" in f["word"].lower() for f in gerunds)

    def test_participle_not_in_phrase_overlap(self):
        # If a word is already matched as a phrase, it should not also
        # be flagged as a participle
        text = "Принимая во внимание факторы."
        findings = self.layer.analyze(text)
        # "Принимая во внимание" is a phrase (Канцеляризм) — "Принимая"
        # should not also appear as a separate gerund
        gerunds = [f for f in findings if f["rule"] == "Возможно, деепричастие"]
        assert not any("принимая" in f["word"].lower() for f in gerunds)

    def test_detects_okazyvat_lemma(self):
        # "оказывают" should be detected via "оказывать" lemma
        findings = self.layer.analyze("Мы оказывают помощь.")
        words = [f["word"].lower() for f in findings]
        assert "оказывают" in words

    def test_mixed_problems_case(self):
        # BAD case #05: should find at least 4 issues
        text = "Высококвалифицированные специалисты нашей динамично развивающейся компании оказывают широкий спектр услуг."
        findings = self.layer.analyze(text)
        rules = [f["rule"] for f in findings]
        assert "Корпоративный штамп" in rules
        assert "Фраза с отглагольным существительным" in rules
        assert len(findings) >= 4



def merge_findings(fast: list[dict], deep: list[dict]) -> list[dict]:
    """Merge findings from fast and deep layers, resolving overlaps.

    When two findings overlap in position, keep the one with higher weight.
    Non-overlapping findings are all kept.
    """
    all_findings = fast + deep
    if not all_findings:
        return []

    # Sort by start position, then by weight descending
    all_findings.sort(key=lambda f: (f["start"], -f.get("weight", 0)))

    result = []
    for finding in all_findings:
        overlapping = False
        for existing in result:
            if finding["start"] < existing["end"] and finding["end"] > existing["start"]:
                # Overlap found — keep higher weight
                if finding.get("weight", 0) > existing.get("weight", 0):
                    result.remove(existing)
                    result.append(finding)
                overlapping = True
                break
        if not overlapping:
            result.append(finding)

    return sorted(result, key=lambda f: f["start"])



from krrkt.engine.dedup import merge_findings


class TestMergeFindings:
    def test_no_overlap(self):
        fast = [{"start": 0, "end": 5, "weight": 100, "source": "fast"}]
        deep = [{"start": 10, "end": 15, "weight": 80, "source": "deep"}]
        result = merge_findings(fast, deep)
        assert len(result) == 2

    def test_exact_overlap_keeps_higher_weight(self):
        fast = [{"start": 0, "end": 5, "weight": 50, "source": "fast"}]
        deep = [{"start": 0, "end": 5, "weight": 100, "source": "deep"}]
        result = merge_findings(fast, deep)
        assert len(result) == 1
        assert result[0]["weight"] == 100

    def test_partial_overlap_keeps_higher_weight(self):
        fast = [{"start": 0, "end": 8, "weight": 100, "source": "fast"}]
        deep = [{"start": 3, "end": 10, "weight": 50, "source": "deep"}]
        result = merge_findings(fast, deep)
        assert len(result) == 1
        assert result[0]["weight"] == 100

    def test_empty_inputs(self):
        assert merge_findings([], []) == []

    def test_one_side_empty(self):
        fast = [{"start": 0, "end": 5, "weight": 100, "source": "fast"}]
        assert len(merge_findings(fast, [])) == 1
        assert len(merge_findings([], fast)) == 1

    def test_sorted_by_position(self):
        fast = [{"start": 20, "end": 25, "weight": 100, "source": "fast"}]
        deep = [{"start": 5, "end": 10, "weight": 80, "source": "deep"}]
        result = merge_findings(fast, deep)
        assert result[0]["start"] < result[1]["start"]



import pytest
from krrkt.engine.scorer import calculate_score, score_color, count_words

class TestCountWords:
    def test_simple_russian(self):
        assert count_words("Мы выпускаем подшипники") == 3

    def test_empty(self):
        assert count_words("") == 0

    def test_mixed_languages(self):
        assert count_words("Claude Code is great инструмент") == 5

    def test_with_punctuation(self):
        assert count_words("Привет, мир! Как дела?") == 4

    def test_hyphenated(self):
        assert count_words("Санкт-Петербург красив") == 2

class TestCalculateScore:
    def test_clean_text_no_findings(self):
        score = calculate_score("Выпускаем подшипники каждый день", [])
        assert score == 10

    def test_empty_text(self):
        assert calculate_score("", []) == 0

    def test_heavily_polluted(self):
        findings = [{"weight": 100}, {"weight": 100}, {"weight": 100}]
        score = calculate_score("Один два три четыре", findings)
        assert score < 2

    def test_half_polluted(self):
        findings = [{"weight": 100}, {"weight": 100}]
        score = calculate_score("Один два три четыре", findings)
        assert score == 1.2

    def test_weight_zero_no_effect(self):
        findings_zero = [{"weight": 0}]
        findings_hundred = [{"weight": 100}]
        text = "Один два три четыре пять"
        assert calculate_score(text, findings_zero) > calculate_score(text, findings_hundred)

    def test_weight_50_counts_half(self):
        findings = [{"weight": 50}]
        text = "Один два три четыре пять"
        assert calculate_score(text, findings) == 7.2

    def test_score_clamped_to_0(self):
        findings = [{"weight": 100}] * 20
        assert calculate_score("Одно слово", findings) >= 0

    def test_score_integer_display(self):
        score = calculate_score("Выпускаем подшипники каждый день", [])
        assert score == 10

class TestScoreColor:
    def test_green(self):
        assert score_color(7.5) == "green"
        assert score_color(10) == "green"

    def test_orange(self):
        assert score_color(5) == "orange"
        assert score_color(7.4) == "orange"

    def test_red(self):
        assert score_color(4.9) == "red"
        assert score_color(0) == "red"



import pytest
from unittest.mock import patch, AsyncMock
from krrkt.engine.pipeline import proofread
from krrkt.models import ProofreadResult


class TestPipeline:
    @pytest.mark.asyncio
    async def test_empty_text(self):
        result = await proofread("", skip_llm=True)
        assert isinstance(result, ProofreadResult)
        assert result.score == 0
        assert result.words == 0
        assert result.findings == []

    @pytest.mark.asyncio
    async def test_clean_text_skip_llm(self):
        result = await proofread("Завод выпускает подшипники.", skip_llm=True)
        assert isinstance(result, ProofreadResult)
        assert result.score >= 8
        assert result.words > 0
        assert result.sentences >= 1

    @pytest.mark.asyncio
    async def test_dirty_text_skip_llm(self):
        result = await proofread(
            "На сегодняшний день наша компания является лидером рынка и предлагает широкий спектр услуг.",
            skip_llm=True
        )
        assert result.score < 8
        assert result.stopwords > 0
        assert len(result.findings) > 0

    @pytest.mark.asyncio
    async def test_findings_have_positions(self):
        result = await proofread("Широкий спектр услуг.", skip_llm=True)
        for f in result.findings:
            assert f.start >= 0
            assert f.end > f.start
            assert f.word == "Широкий спектр услуг."[f.start:f.end]

    @pytest.mark.asyncio
    async def test_summary_has_sections(self):
        result = await proofread(
            "Данная уникальная компания является лидером рынка.",
            skip_llm=True
        )
        if result.stopwords > 0:
            assert len(result.summary) > 0

    @pytest.mark.asyncio
    async def test_result_has_score_color(self):
        result = await proofread("Текст для проверки.", skip_llm=True)
        assert result.score_color in ("green", "orange", "red")



from abc import ABC, abstractmethod


class LLMProvider(ABC):
    @abstractmethod
    async def analyze(self, text: str, system_prompt: str,
                      already_found: list[dict]) -> list[dict]:
        """Analyze text and return list of findings.

        Each finding is a dict with: start, end, word, rule, section,
        description, weight, style, tab, source="deep"
        """
        ...



import re
from krrkt.rules.loader import load_patterns

try:
    import pymorphy3
    _morph = pymorphy3.MorphAnalyzer()
except ImportError:
    _morph = None


# Common short past participles that are part of passive voice constructions.
# These should be caught by phrase-level "было + participle" rules, not the
# automatic participle detector, to avoid double-flagging / wrong category.
PASSIVE_PARTICIPLES = {
    "принято", "принят", "принята", "приняты",
    "решено", "сделано", "получено",
    "установлено", "проведено", "создано", "обнаружено",
    "выполнено", "достигнуто", "завершено", "направлено",
    "основано", "организовано", "рассмотрено",
}


class FastLayer:
    def __init__(self):
        patterns = load_patterns()
        self.phrases = sorted(
            patterns.get("phrases", []),
            key=lambda p: len(p["pattern"]),
            reverse=True  # longest first
        )
        self.lemmas = patterns.get("lemmas", [])
        self.regex_patterns = [
            {**p, "_compiled": re.compile(p["pattern"], re.IGNORECASE | re.UNICODE)}
            for p in patterns.get("regex", [])
        ]

    def analyze(self, text: str) -> list[dict]:
        findings = []
        used_ranges = set()

        # Phase 1: Phrase matching (longest first, case-insensitive)
        text_lower = text.lower()
        for phrase_def in self.phrases:
            pattern = phrase_def["pattern"].lower()
            start = 0
            while True:
                idx = text_lower.find(pattern, start)
                if idx == -1:
                    break
                end = idx + len(pattern)
                if not self._overlaps(idx, end, used_ranges):
                    findings.append(self._make_finding(text, idx, end, phrase_def))
                    used_ranges.add((idx, end))
                start = idx + 1

        # Phase 2: Lemma matching via pymorphy3
        if _morph:
            for token_match in re.finditer(
                r'[A-Za-z\u0410-\u044f\u0401\u0451][\w-]*', text, re.UNICODE
            ):
                token = token_match.group()
                token_start = token_match.start()
                token_end = token_match.end()
                if self._overlaps(token_start, token_end, used_ranges):
                    continue
                parsed = _morph.parse(token)
                if not parsed:
                    continue
                lemma = parsed[0].normal_form
                for lemma_def in self.lemmas:
                    if lemma == lemma_def["lemma"].lower():
                        findings.append(self._make_finding(text, token_start, token_end, lemma_def))
                        used_ranges.add((token_start, token_end))
                        break

        # Phase 2.5: Detect participles and gerunds via morphology
        if _morph:
            for token_match in re.finditer(r'[А-Яа-яЁё][\w-]*', text, re.UNICODE):
                token = token_match.group()
                token_start = token_match.start()
                token_end = token_match.end()
                if self._overlaps(token_start, token_end, used_ranges):
                    continue
                parsed = _morph.parse(token)
                if not parsed:
                    continue
                tag = parsed[0].tag
                # Participle (причастие): PRTF (full) or PRTS (short)
                # Skip common short participles used in passive constructions
                if token.lower() in PASSIVE_PARTICIPLES:
                    continue
                if 'PRTF' in tag or 'PRTS' in tag:
                    findings.append(self._make_finding(text, token_start, token_end, {
                        "rule": "Возможно, причастие",
                        "section": "причастие",
                        "weight": 50,
                        "style": "blue",
                        "tab": "blue",
                        "description": "Постарайтесь заменить на глагол.",
                    }))
                    used_ranges.add((token_start, token_end))
                # Gerund (деепричастие): GRND
                elif 'GRND' in tag:
                    findings.append(self._make_finding(text, token_start, token_end, {
                        "rule": "Возможно, деепричастие",
                        "section": "деепричастие",
                        "weight": 50,
                        "style": "blue",
                        "tab": "blue",
                        "description": "Постарайтесь заменить на глагол.",
                    }))
                    used_ranges.add((token_start, token_end))

        # Phase 3: Regex patterns
        for rx_def in self.regex_patterns:
            for m in rx_def["_compiled"].finditer(text):
                s, e = m.start(), m.end()
                if not self._overlaps(s, e, used_ranges):
                    findings.append(self._make_finding(text, s, e, rx_def))
                    used_ranges.add((s, e))

        return findings

    def _overlaps(self, start: int, end: int, used: set) -> bool:
        for us, ue in used:
            if start < ue and end > us:
                return True
        return False

    def _make_finding(self, text: str, start: int, end: int, rule_def: dict) -> dict:
        return {
            "start": start,
            "end": end,
            "word": text[start:end],
            "rule": rule_def.get("rule", ""),
            "section": rule_def.get("section", ""),
            "description": rule_def.get("description", ""),
            "weight": rule_def.get("weight", 100),
            "style": rule_def.get("style", "red"),
            "tab": rule_def.get("tab", "red"),
            "source": "fast",
        }



"""LLM provider using Claude Agent SDK (works via Claude subscription, no API key needed)."""

import json
from claude_agent_sdk import query, ClaudeAgentOptions, ResultMessage
from krrkt.providers.base import LLMProvider


class AgentSDKProvider(LLMProvider):
    """Uses Claude Agent SDK to analyze text via Claude Code subscription."""

    def __init__(self, model: str = "claude-sonnet-4-6"):
        self.model = model

    async def analyze(self, text: str, system_prompt: str,
                      already_found: list[dict]) -> list[dict]:
        already_desc = ""
        if already_found:
            items = [f'- "{f.get("word", "")}" ({f.get("rule", "")})' for f in already_found]
            already_desc = "\n\nУже найденные проблемы (НЕ дублируй их):\n" + "\n".join(items)

        user_prompt = (
            f"{system_prompt}\n\n---\n\n"
            f"Проанализируй текст и верни ТОЛЬКО JSON в формате "
            f'{{"findings": [{{...}}]}}:\n\n'
            f"{text}{already_desc}"
        )

        result_text = ""
        async for message in query(
            prompt=user_prompt,
            options=ClaudeAgentOptions(
                model=self.model,
                max_turns=1,
            ),
        ):
            if isinstance(message, ResultMessage):
                result_text = message.result

        if not result_text:
            return []

        # Parse JSON from result — may be wrapped in markdown or have text before it
        data = None
        try:
            # Try 1: direct JSON parse
            data = json.loads(result_text.strip())
        except json.JSONDecodeError:
            pass

        if data is None:
            # Try 2: extract from markdown code fence
            import re
            fence_match = re.search(r'```(?:json)?\s*\n(.*?)\n```', result_text, re.DOTALL)
            if fence_match:
                try:
                    data = json.loads(fence_match.group(1))
                except json.JSONDecodeError:
                    pass

        if data is None:
            # Try 3: find first { ... } block
            import re
            brace_match = re.search(r'\{.*"findings".*\}', result_text, re.DOTALL)
            if brace_match:
                try:
                    data = json.loads(brace_match.group(0))
                except json.JSONDecodeError:
                    pass

        if data is None:
            return []

        findings = data.get("findings", [])

        # Normalize and validate
        valid_styles = {"red", "blue", "orange"}
        valid_tabs = {"red", "blue"}

        for f in findings:
            f["source"] = "deep"
            f.setdefault("section", "")
            f.setdefault("description", "")
            # Normalize style/tab to valid values
            if f.get("style") not in valid_styles:
                f["style"] = "orange"
            if f.get("tab") not in valid_tabs:
                f["tab"] = "red"

        return findings



from krrkt.engine.splitter import split_text

class TestSplitText:
    def test_single_sentence(self):
        result = split_text("Привет мир.")
        assert len(result) == 1
        assert result[0]["text"] == "Привет мир."

    def test_two_sentences(self):
        result = split_text("Первое предложение. Второе предложение.")
        assert len(result) == 2

    def test_question_and_exclamation(self):
        result = split_text("Как дела? Отлично! Спасибо.")
        assert len(result) == 3

    def test_preserves_positions(self):
        text = "Первое. Второе."
        result = split_text(text)
        for chunk in result:
            assert text[chunk["start"]:chunk["end"]] == chunk["text"]

    def test_empty_text(self):
        assert split_text("") == []

    def test_whitespace_only(self):
        assert split_text("   ") == []

    def test_no_punctuation(self):
        result = split_text("Текст без точки")
        assert len(result) == 1

    def test_newlines_split(self):
        result = split_text("Первое.\nВторое.")
        assert len(result) == 2

    def test_positions_cover_full_text(self):
        text = "Первое. Второе. Третье."
        result = split_text(text)
        assert result[0]["start"] == 0
        assert result[-1]["end"] == len(text)



import pytest
from krrkt.providers.anthropic_provider import AnthropicProvider
from krrkt.engine.deep_layer import DeepLayer


@pytest.mark.llm
@pytest.mark.asyncio
async def test_llm_finds_issues():
    provider = AnthropicProvider()
    layer = DeepLayer(provider=provider)
    findings = await layer.analyze(
        "Наша компания является лидером рынка и осуществляет деятельность в сфере IT.",
        already_found=[]
    )
    assert isinstance(findings, list)
    assert len(findings) > 0
    assert all(f["source"] == "deep" for f in findings)
    assert all("start" in f and "end" in f for f in findings)


@pytest.mark.llm
@pytest.mark.asyncio
async def test_llm_clean_text():
    provider = AnthropicProvider()
    layer = DeepLayer(provider=provider)
    findings = await layer.analyze(
        "Выпускаем подшипники в Санкт-Петербурге.",
        already_found=[]
    )
    assert isinstance(findings, list)
    # Clean text should have few or no findings
    serious = [f for f in findings if f.get("weight", 0) >= 80]
    assert len(serious) <= 1



import re

SENTENCE_RE = re.compile(
    r'[.?!](\s*\r?\n|\s+(?=[A-Z\u0410-\u042f]))'
)
SIZE_LIMIT = 50000


def split_text(text: str) -> list[dict]:
    """Split text into sentence chunks with positions.

    Returns list of {"text": str, "start": int, "end": int}.
    Based on Glvrd's SplitText logic.
    """
    if not text or not text.strip():
        return []

    chunks = []
    start = 0

    for m in SENTENCE_RE.finditer(text):
        end = m.end()
        if end > SIZE_LIMIT:
            break
        chunks.append({
            "text": text[start:end],
            "start": start,
            "end": end,
        })
        start = end

    # Remainder
    if start < len(text) and len(text) <= SIZE_LIMIT:
        remainder = text[start:]
        if remainder.strip():
            chunks.append({
                "text": remainder,
                "start": start,
                "end": len(text),
            })

    # If no splits found, return whole text
    if not chunks and text.strip():
        chunks.append({"text": text, "start": 0, "end": len(text)})

    return chunks



from dataclasses import dataclass, field


@dataclass
class Finding:
    start: int
    end: int
    word: str
    rule: str
    section: str = ""
    description: str = ""
    weight: int = 100
    style: str = "red"
    tab: str = "red"
    source: str = "fast"


@dataclass
class ProofreadResult:
    score: float
    score_color: str
    text: str
    words: int
    sentences: int
    findings: list[Finding]
    stopwords: int
    summary: dict = field(default_factory=dict)



from krrkt.rules.loader import load_rules, load_patterns, load_config

class TestLoadRules:
    def test_loads_rules_dict(self):
        rules = load_rules()
        assert isinstance(rules, dict)
        assert "categories" in rules
        assert len(rules["categories"]) > 0

    def test_rule_has_required_fields(self):
        rules = load_rules()
        for cat in rules["categories"]:
            assert "name" in cat
            assert "weight" in cat
            assert "style" in cat
            assert "tab" in cat
            assert "description" in cat

    def test_has_known_categories(self):
        rules = load_rules()
        names = [c["name"] for c in rules["categories"]]
        assert "Канцеляризм" in names
        assert "Усилитель" in names
        assert "Корпоративный штамп" in names

class TestLoadPatterns:
    def test_loads_patterns(self):
        patterns = load_patterns()
        assert "phrases" in patterns
        assert "lemmas" in patterns
        assert len(patterns["phrases"]) > 0
        assert len(patterns["lemmas"]) > 0

    def test_phrase_has_required_fields(self):
        patterns = load_patterns()
        for p in patterns["phrases"]:
            assert "pattern" in p
            assert "rule" in p
            assert "weight" in p

    def test_lemma_has_required_fields(self):
        patterns = load_patterns()
        for l in patterns["lemmas"]:
            assert "lemma" in l
            assert "rule" in l
            assert "weight" in l

    def test_has_regex(self):
        patterns = load_patterns()
        assert "regex" in patterns
        assert len(patterns["regex"]) > 0

class TestLoadConfig:
    def test_loads_config(self):
        config = load_config()
        assert "model" in config
        assert "server" in config
        assert "scoring" in config

    def test_default_model(self):
        config = load_config()
        assert config["model"] == "claude-sonnet-4-6"



from pathlib import Path
import yaml
from krrkt.providers.base import LLMProvider


def _load_system_prompt() -> str:
    # Package data first, then dev config
    prompt_file = Path(__file__).parent.parent / "data" / "prompts" / "analyze.yaml"
    if not prompt_file.exists():
        prompt_file = Path(__file__).parent.parent.parent.parent / "config" / "prompts" / "analyze.yaml"
    with open(prompt_file, encoding="utf-8") as f:
        data = yaml.safe_load(f)
    return data.get("system_prompt", "")


class DeepLayer:
    def __init__(self, provider: LLMProvider):
        self.provider = provider
        self.system_prompt = _load_system_prompt()

    async def analyze(self, text: str, already_found: list[dict] | None = None,
                      model: str | None = None) -> list[dict]:
        if already_found is None:
            already_found = []
        return await self.provider.analyze(text, self.system_prompt, already_found)



import math
import re

WORDS_REGEX = re.compile(r'[A-Za-z\u0410-\u044f\u0401\u0451\d-]+([^A-Za-z\u0410-\u044f\u0401\u0451\d-]+)?')

def count_words(text: str) -> int:
    """Count words in text using Glvrd's regex."""
    if not text:
        return 0
    return len(WORDS_REGEX.findall(text.strip()))

def calculate_score(text: str, findings: list[dict]) -> int | float:
    """Calculate Glvrd score using cubic formula.

    Formula: floor((1 - weighted_stopwords / total_words)^3 * 100) / 10
    Each finding contributes weight/100 to weighted_stopwords.
    """
    words = count_words(text)
    if words == 0:
        return 0
    stopwords = sum(f.get("weight", 100) / 100 for f in findings)
    penalties = sum(f.get("penalty", 0) for f in findings)
    score_raw = math.floor((1 - stopwords / words) ** 3 * 100) - penalties
    score_raw = max(0, min(100, score_raw))
    if score_raw % 10 == 0:
        return score_raw // 10
    return round(score_raw / 10, 1)

def score_color(score: float) -> str:
    if score >= 7.5:
        return "green"
    if score >= 5:
        return "orange"
    return "red"



import pytest
from fastapi.testclient import TestClient
from krrkt.api.app import app


client = TestClient(app)


class TestHealthEndpoint:
    def test_health(self):
        response = client.get("/api/v1/health")
        assert response.status_code == 200
        assert response.json()["status"] == "ok"


class TestRulesEndpoint:
    def test_returns_rules(self):
        response = client.get("/api/v1/rules")
        assert response.status_code == 200
        data = response.json()
        assert "categories" in data


class TestProofreadEndpoint:
    def test_proofread_clean_text(self):
        response = client.post("/api/v1/proofread", json={
            "text": "Завод выпускает подшипники.",
            "skip_llm": True,
        })
        assert response.status_code == 200
        data = response.json()
        assert "score" in data
        assert "findings" in data
        assert data["score"] >= 8

    def test_proofread_dirty_text(self):
        response = client.post("/api/v1/proofread", json={
            "text": "Наша компания является лидером рынка и предлагает широкий спектр услуг.",
            "skip_llm": True,
        })
        assert response.status_code == 200
        data = response.json()
        assert data["score"] < 8
        assert len(data["findings"]) > 0

    def test_proofread_empty_text(self):
        response = client.post("/api/v1/proofread", json={
            "text": "",
            "skip_llm": True,
        })
        assert response.status_code == 200
        assert response.json()["score"] == 0

    def test_findings_structure(self):
        response = client.post("/api/v1/proofread", json={
            "text": "Широкий спектр услуг высочайшего качества.",
            "skip_llm": True,
        })
        data = response.json()
        for f in data["findings"]:
            assert "start" in f
            assert "end" in f
            assert "word" in f
            assert "rule" in f
            assert "weight" in f
            assert "source" in f



import json
import anthropic
from krrkt.providers.base import LLMProvider


FINDINGS_SCHEMA = {
    "type": "object",
    "properties": {
        "findings": {
            "type": "array",
            "items": {
                "type": "object",
                "properties": {
                    "start": {"type": "integer", "description": "Start position in original text"},
                    "end": {"type": "integer", "description": "End position in original text"},
                    "word": {"type": "string", "description": "The problematic word or phrase"},
                    "rule": {"type": "string", "description": "Rule category name"},
                    "section": {"type": "string", "description": "Section for grouping"},
                    "description": {"type": "string", "description": "Brief recommendation"},
                    "weight": {"type": "integer", "description": "Severity 0-100"},
                    "style": {"type": "string", "enum": ["red", "blue", "orange"]},
                    "tab": {"type": "string", "enum": ["red", "blue"]},
                },
                "required": ["start", "end", "word", "rule", "weight", "style", "tab"],
                "additionalProperties": False,
            }
        }
    },
    "required": ["findings"],
    "additionalProperties": False,
}


class AnthropicProvider(LLMProvider):
    def __init__(self, model: str = "claude-sonnet-4-6"):
        self.client = anthropic.Anthropic()
        self.model = model

    async def analyze(self, text: str, system_prompt: str,
                      already_found: list[dict]) -> list[dict]:
        already_desc = ""
        if already_found:
            items = [f"- \"{f.get('word', '')}\" ({f.get('rule', '')})" for f in already_found]
            already_desc = "\n\nУже найденные проблемы (НЕ дублируй их):\n" + "\n".join(items)

        user_content = f"Проанализируй текст:\n\n{text}{already_desc}"

        response = self.client.messages.create(
            model=self.model,
            max_tokens=4096,
            system=[{
                "type": "text",
                "text": system_prompt,
                "cache_control": {"type": "ephemeral"},
            }],
            messages=[{"role": "user", "content": user_content}],
            output_config={
                "format": {
                    "type": "json_schema",
                    "schema": FINDINGS_SCHEMA,
                }
            },
        )

        text_block = next(
            (b.text for b in response.content if b.type == "text"), "{}"
        )
        data = json.loads(text_block)
        findings = data.get("findings", [])

        # Ensure source is set
        for f in findings:
            f["source"] = "deep"
            f.setdefault("section", "")
            f.setdefault("description", "")

        return findings



from pathlib import Path
import yaml

# Package data directory (installed via pip/uvx)
_PACKAGE_DATA = Path(__file__).parent.parent / "data"
# Development config directory (repo root/config)
_DEV_CONFIG = Path(__file__).parent.parent.parent.parent / "config"


def _load_yaml(filename: str) -> dict:
    for base in (_PACKAGE_DATA, _DEV_CONFIG):
        filepath = base / filename
        if filepath.exists():
            with open(filepath, encoding="utf-8") as f:
                return yaml.safe_load(f)
    raise FileNotFoundError(
        f"Config file not found: {filename} "
        f"(searched {_PACKAGE_DATA} and {_DEV_CONFIG})"
    )


def load_rules() -> dict:
    return _load_yaml("rules.yaml")


def load_patterns() -> dict:
    return _load_yaml("patterns.yaml")


def load_config() -> dict:
    return _load_yaml("config.yaml")



import pytest
from krrkt.engine.pipeline import proofread


class TestPipelineFastOnly:
    @pytest.mark.asyncio
    async def test_corporate_stamps_detected(self):
        result = await proofread(
            "Наша компания предлагает широкий спектр услуг высочайшего качества.",
            skip_llm=True
        )
        rules = [f.rule for f in result.findings]
        assert any("штамп" in r.lower() or "оценка" in r.lower() for r in rules)

    @pytest.mark.asyncio
    async def test_bureaucratic_text(self):
        result = await proofread(
            "В связи с вышеизложенным, принимая во внимание все факторы, считаем целесообразным принять решение.",
            skip_llm=True
        )
        assert result.score < 7
        rules = [f.rule for f in result.findings]
        assert "Канцеляризм" in rules

    @pytest.mark.asyncio
    async def test_pleonasm_detected(self):
        result = await proofread("Это бесплатный подарок для клиента.", skip_llm=True)
        rules = [f.rule for f in result.findings]
        assert "Плеоназм" in rules



import pytest
import yaml
from pathlib import Path
from krrkt.engine.pipeline import proofread


FIXTURES_PATH = Path(__file__).parent.parent / "fixtures" / "texts.yaml"


def load_golden_tests():
    with open(FIXTURES_PATH, encoding="utf-8") as f:
        data = yaml.safe_load(f)
    return data["tests"]


golden_tests = load_golden_tests()


@pytest.mark.parametrize(
    "test_case",
    golden_tests,
    ids=[t["name"] for t in golden_tests],
)
@pytest.mark.asyncio
async def test_golden_score_range(test_case):
    """Verify score falls within expected range."""
    result = await proofread(test_case["text"], skip_llm=True)

    score_min = test_case.get("expected_score_min", 0)
    score_max = test_case.get("expected_score_max", 10)
    assert score_min <= result.score <= score_max, (
        f"Score {result.score} not in [{score_min}, {score_max}] for '{test_case['name']}'"
    )


@pytest.mark.parametrize(
    "test_case",
    [t for t in golden_tests if "must_find_rules" in t],
    ids=[t["name"] for t in golden_tests if "must_find_rules" in t],
)
@pytest.mark.asyncio
async def test_golden_must_find_rules(test_case):
    """Verify expected rule categories are detected."""
    result = await proofread(test_case["text"], skip_llm=True)
    found_rules = {f.rule for f in result.findings}

    for expected_rule in test_case["must_find_rules"]:
        assert expected_rule in found_rules, (
            f"Expected rule '{expected_rule}' not found in {found_rules} for '{test_case['name']}'"
        )


@pytest.mark.parametrize(
    "test_case",
    [t for t in golden_tests if "must_find" in t],
    ids=[t["name"] for t in golden_tests if "must_find" in t],
)
@pytest.mark.asyncio
async def test_golden_must_find_words(test_case):
    """Verify specific words/phrases are detected."""
    result = await proofread(test_case["text"], skip_llm=True)
    found_words = {f.word.lower() for f in result.findings}

    for expected_word in test_case["must_find"]:
        assert any(expected_word.lower() in w for w in found_words), (
            f"Expected word '{expected_word}' not found in {found_words} for '{test_case['name']}'"
        )


@pytest.mark.parametrize(
    "test_case",
    [t for t in golden_tests if "must_not_find_rules" in t],
    ids=[t["name"] for t in golden_tests if "must_not_find_rules" in t],
)
@pytest.mark.asyncio
async def test_golden_must_not_find_rules(test_case):
    """Verify certain rules are NOT triggered for clean text."""
    result = await proofread(test_case["text"], skip_llm=True)
    found_rules = {f.rule for f in result.findings}

    for unwanted_rule in test_case["must_not_find_rules"]:
        assert unwanted_rule not in found_rules, (
            f"Unexpected rule '{unwanted_rule}' found for '{test_case['name']}'"
        )



import pytest
from unittest.mock import AsyncMock
from krrkt.engine.deep_layer import DeepLayer


class TestDeepLayer:
    @pytest.mark.asyncio
    async def test_returns_findings_list(self):
        mock_provider = AsyncMock()
        mock_provider.analyze.return_value = [
            {"start": 0, "end": 5, "word": "Очень", "rule": "Усилитель",
             "weight": 100, "style": "red", "tab": "red",
             "description": "Замените фактами", "section": "усилитель",
             "source": "deep"}
        ]
        layer = DeepLayer(provider=mock_provider)
        findings = await layer.analyze("Очень хороший текст.", already_found=[])
        assert len(findings) == 1
        assert findings[0]["source"] == "deep"

    @pytest.mark.asyncio
    async def test_passes_already_found(self):
        mock_provider = AsyncMock()
        mock_provider.analyze.return_value = []
        layer = DeepLayer(provider=mock_provider)
        already = [{"start": 0, "end": 5, "word": "Очень", "rule": "Усилитель"}]
        await layer.analyze("Очень хороший текст.", already_found=already)
        call_args = mock_provider.analyze.call_args
        assert call_args[0][2] == already  # third positional arg is already_found

    @pytest.mark.asyncio
    async def test_empty_text(self):
        mock_provider = AsyncMock()
        mock_provider.analyze.return_value = []
        layer = DeepLayer(provider=mock_provider)
        findings = await layer.analyze("", already_found=[])
        assert findings == []

    @pytest.mark.asyncio
    async def test_none_already_found(self):
        mock_provider = AsyncMock()
        mock_provider.analyze.return_value = []
        layer = DeepLayer(provider=mock_provider)
        await layer.analyze("Текст для проверки.")
        call_args = mock_provider.analyze.call_args
        assert call_args[0][2] == []  # defaults to empty list
